import {
	application,
	DataSource,
	Resource,
	resource,
	valibot,
	dataSources,
	serializers
} from '.';

const autoIncrement = async (dataSource: DataSource) => {
	const data = await dataSource.getMultiple() as Record<string, string>[];

	const highestId = data.reduce<number>(
		(highestId, d) => (Number(d.id) > highestId ? Number(d.id) : highestId),
		-Infinity
	);

	return (highestId + 1).toString();
};


const TEXT_SERIALIZER_PAIR = {
	serialize(obj: unknown, level = 0): string {
		if (Array.isArray(obj)) {
			return obj.map((o) => this.serialize(o)).join('\n\n');
		}

		if (typeof obj === 'object') {
			if (obj !== null) {
				return Object.entries(obj)
					.map(([key, value]) => `${Array(level * 2).fill(' ').join('')}${key}: ${this.serialize(value, level + 1)}`)
					.join('\n');
			}

			return '';
		}

		if (typeof obj === 'number' && Number.isFinite(obj)) {
			return obj.toString();
		}

		if (typeof obj === 'string') {
			return obj;
		}

		return '';
	},
	deserialize: <T>(str: string) => str as T
};

const Piano = resource(valibot.object(
	{
		brand: valibot.string()
	},
	valibot.never()
))
	.name('Piano')
	.id('id', {
		generationStrategy: autoIncrement,
	});

// TODO implement authentication and RBAC on each resource

const User = resource(valibot.object(
	{
		firstName: valibot.string(),
		middleName: valibot.string(),
		lastName: valibot.string(),
		bio: valibot.string(),
		createdAt: valibot.date()
	},
	valibot.never()
))
	.name('User')
	.fullText('bio')
	.id('id', {
		generationStrategy: autoIncrement,
	});

const app = application({
	name: 'piano-service',
	dataSource: (resource: Resource) => new dataSources.jsonlFile.DataSource(resource),
})
	.contentType('application/json', serializers.applicationJson)
	.contentType('text/json', serializers.textJson)
	.contentType('text/plain', TEXT_SERIALIZER_PAIR)
	.resource(Piano)
	.resource(User);

const server = app.createServer({
	baseUrl: '/api'
});

server.listen(3000);

