import { IncomingMessage } from 'http';
import { SerializerPair } from './serializers';
import { BaseSchema, parseAsync } from 'valibot';

export const getMethod = (req: IncomingMessage) => req.method!.toUpperCase();

export const getUrl = (req: IncomingMessage, baseUrl?: string) => {
	const urlObject = new URL(req.url!, 'http://localhost');
	const urlWithoutBaseRaw = urlObject.pathname.slice(baseUrl?.length ?? 0);

	return {
		url: urlWithoutBaseRaw.length < 1 ? '/' : urlWithoutBaseRaw,
		query: urlObject.searchParams
	};
}

export const getBody = (req: IncomingMessage, deserializer: SerializerPair, schema: BaseSchema) => {
	return new Promise((resolve, reject) => {
		let body = Buffer.from('');
		req.on('data', (chunk) => {
			body = Buffer.concat([body, chunk]);
		});
		req.on('end', async () => {
			const bodyStr = body.toString('utf-8'); // TODO use encoding in request header
			try {
				const bodyDeserialized = await parseAsync(
					schema,
					deserializer.deserialize(bodyStr),
					{ abortEarly: false }
				);
				resolve(bodyDeserialized);
			} catch (err) {
				reject(err);
			}
		});
	});
};
